@@ -231,7 +231,7 @@ int mmc_init(mmap_cache * cache) {
   ASSERT(c_num_pages >= 1 && c_num_pages <= 1000);
 
   c_page_size = cache->c_page_size;
-  ASSERT(c_page_size >= 1024 && c_page_size <= 1024*1024);
+  ASSERT(c_page_size >= 1024 && c_page_size <= 16*1024*1024);
 
   start_slots = cache->start_slots;
   ASSERT(start_slots >= 10 && start_slots <= 500);
@@ -417,6 +417,10 @@ int mmc_lock(mmap_cache * cache, MU32 p_cur) {
   int lock_res = -1;
   void * p_ptr;
 
+  /* Check not already locked */
+  if (cache->p_cur != -1)
+    return -1 + _mmc_set_error(cache, 0, "page %u is already locked, can't lock multiple pages", cache->p_cur);
+
   /* Setup page details */
   p_offset = p_cur * cache->c_page_size;
   p_ptr = PTR_ADD(cache->mm_var, p_offset);
@@ -631,7 +635,7 @@ int mmc_read(
  *   cache_mmap * cache, MU32 hash_slot,
  *   void *key_ptr, int key_len,
  *   void *val_ptr, int val_len,
- *   MU32 flags
+ *   MU32 expire_seconds, MU32 flags
  * )
  *
  * Write key to current page
@@ -641,7 +645,7 @@ int mmc_write(
   mmap_cache *cache, MU32 hash_slot,
   void *key_ptr, int key_len,
   void *val_ptr, int val_len,
-  MU32 flags
+  MU32 expire_seconds, MU32 flags
 ) {
   int did_store = 0;
   MU32 kvlen = KV_SlotLen(key_len, val_len);
@@ -671,7 +675,8 @@ int mmc_write(
     MU32 now = (MU32)time(0);
 
     /* Calculate expiry time */
-    MU32 expire_time = cache->expire_time ? now + cache->expire_time : 0;
+    if (expire_seconds == (MU32)-1) expire_seconds = cache->expire_time;
+    MU32 expire_time = expire_seconds ? now + expire_seconds : 0;
 
     /* Store info into slot */
     S_LastAccess(base_det) = now;
